#!/usr/bin/python
from BaseHTTPServer import BaseHTTPRequestHandler,HTTPServer
from os import curdir, sep, system
from urlparse import urlparse, parse_qs
import pigpio
import threading
import Queue
import time

PORT_NUMBER = 80

system("sudo pigpiod -s2 -b1000")
time.sleep(5)

class angstromHandler():

	# LED to GPIO pin mappings

	UV=17
	V=27
	RB=22
	B=10
	C=9
	G=4
	Y=5
	A=6
	R=13
	DR=19
	IR=26
	YG=11

	# PWM frequency
	PWM_FREQ=3200

	pi = pigpio.pi()

	lock = threading.Lock()

	ledList = [["UV",UV],["V",V],["RB",RB],["B",B],["C",C],["G",G],["Y",Y],["A",A],["R",R],["DR",DR],["IR",IR],["YG",YG]]

	workerThread = None

	workerQueue = Queue.Queue()

	def worker(self):
		while 1==1:
			val = self.workerQueue.get()
			#print "dequeue"
			self.setLed(val[0], val[1])



	def __init__(self):
		self.setPWMFreq()
		self.setAllLeds(0)
		self.workerThread = threading.Thread(target=self.worker)
		self.workerThread.daemon = True
		self.workerThread.start()

	def getColour(self, colourString):
		for led in self.ledList:
			if led[0] == colourString:
				return led[1]

	def enqueueLedChanges(self, colourStrings,intensities):
		intensityValues = intensities.split(",")
		colourNames = colourStrings.split(",")
		i = 0
		for intensityVal in intensityValues:
			intensity=int(intensityVal)
			colourString = colourNames[i]
			i = i + 1
			colour = self.getColour(colourString)
			self.workerQueue.put([colour,intensity])

# pigpio has thread limits so serialise calls
	def setLed(self, colour, intensity):
		with self.lock:
			pi = self.pi
			pi.set_PWM_dutycycle(colour, intensity)

	def setAllLeds(self, intensity):
		for led in self.ledList:			
			self.setLed(led[1], intensity)

	def setPWMFreq(self):
		pi = self.pi
		for led in self.ledList:
			pi.set_PWM_frequency(led[1],self.PWM_FREQ)


#This class will handles any incoming request from
#the browser 
class myHandler(BaseHTTPRequestHandler):


	
	angstrom = angstromHandler()
	
	
	def processSliderChange(self):
		url = self.path
		#print "slider change"
		parsed_url = urlparse(url)
		params = parse_qs(parsed_url.query)
		#print int(params["val"][0])
		self.angstrom.enqueueLedChanges(params["id"][0],params["val"][0])

	def sendEmptyResponse(self):
		self.send_response(200)
		self.send_header('Content-type',"text/plain")
		self.end_headers()
		self.wfile.write("OK")

	#Handler for the GET requests
	def do_GET(self):
		print self.path
		if self.path=="/":
			self.path="/static.html"

		try:
			#Check the file extension required and
			#set the right mime type

			sendReply = False
			if self.path.startswith("/sliderchange"):
				self.processSliderChange()
				self.sendEmptyResponse()
				return

			if self.path.startswith("/allledsoff"):
				self.angstrom.setAllLeds(0)
				self.sendEmptyResponse()
				return	

			if self.path.endswith(".html"):
				mimetype='text/html'
				sendReply = True
			if self.path.endswith(".jpg"):
				mimetype='image/jpg'
				sendReply = True
			if self.path.endswith(".gif"):
				mimetype='image/gif'
				sendReply = True
			if self.path.endswith(".js"):
				mimetype='application/javascript'
				sendReply = True
			if self.path.endswith(".css"):
				mimetype='text/css'
				sendReply = True

			if self.path.endswith(".woff2"):
				mimetype="font/woff2"
				sendReply = True	

			if self.path.endswith(".woff"):
				mimetype="font/woff"
				sendReply = True	

			if self.path.endswith(".ttf"):
				mimetype="font/ttf"
				sendReply = True	


			if sendReply == True:
				#Open the static file requested and send it
				f = open(curdir + sep + self.path) 
				self.send_response(200)
				self.send_header('Content-type',mimetype)
				self.end_headers()
				self.wfile.write(f.read())
				f.close()
			return


		except IOError:
			self.send_error(404,'File Not Found: %s' % self.path)

try:
	#Create a web server and define the handler to manage the
	#incoming request

	server = HTTPServer(('raspberrypi.local', PORT_NUMBER), myHandler)
	print 'Started httpserver on port ' , PORT_NUMBER
	
	#Wait forever for incoming htto requests
	server.serve_forever()

except KeyboardInterrupt:
	print '^C received, shutting down the web server'
	server.socket.close()


